import 'package:organic_injector/src/injectables/injectable.dart';
import 'package:organic_injector/src/mapping/injection_mapping.dart';
import 'package:organic_injector/src/providers/i_provider.dart';


abstract class IInjector {
  ///
  /// map type to a already instantiated value.
  ///
  ///     injector.mapToValue(Counter, new CounterModel());
  ///
  /// Optionally add name.
  ///
  ///    injector.mapToValue(ChangeNotifier,new ChangeNotifier(), name: 'counter_changed');
  ///
  /// If you want to provide dependecies of previously mapped values to your mapping, implement IInjectable into the class.
  /// Injector will then run [inject] with an instance of the injector, so you can chain your mappings.
  ///
  void mapToValue(Type type, Object value, {String name: '', bool injectInto:true});


  /// map type to a provider that implements IProvider.
  /// It should return one instance of the Type every time .apply(...) is called on the provider
  /// We use it as a factory for returning new instances on inject
  ///
  /// see [IProvider]
  void mapToProvider(Type type, IProvider provider,{String name: ''});
  ///
  /// Injector will return instance that was previously mapped
  ///
  /// TimerModel model = injector.getInstance(TimerModel);
  ///
  /// StreamController<int> streamController = injector.getInstance(StreamController,'counter_updated');
  Object getInstance(Type type, {String name = ''});

  ///
  /// provide dependecies for an object based on the IInjectable rules
  void injectInto(IInjectable object);

  set parentInjector(IInjector value);

  IInjector get parentInjector;

  void unmap(Type type, [String name = '']);

  bool satisfies(Type type, [String name = '']);

  void teardown();

  IInjector createChildInjector();

  bool hasMapping(Type type, [String name = '']);

  IProvider _getProvider(String _mappingId);

  InjectionMapping _createMapping(String mappingId, Type type, String name);





}