import 'package:organic_injector/src/errors/injector_error.dart';
import 'package:organic_injector/src/injector.dart';
import 'package:organic_injector/src/providers/i_provider.dart';
import 'package:organic_injector/src/providers/value_provider.dart';

class InjectionMapping {

  /*Public fields*/
  bool get isSealed => _sealed;

  bool get hasProvider => _creatingInjector.providerMappings[_mappingId] != null;

  /*Private fields*/
  Type _type;

  String _name;

  String _mappingId;

  Injector _creatingInjector;

  IProvider provider;

  bool _sealed = false;

  Symbol _sealKey;

  /*Constructor*/
  InjectionMapping(this._mappingId, this._creatingInjector, this._type,
      this._name);

  /*public methods*/
  Symbol seal() {
    if (_sealed) {
      throw new InjectorError('Mapping is already sealed.');
    }
    _sealed = true;
    _sealKey = new Symbol("sealed");
    return _sealKey;
  }

  InjectionMapping unseal(Symbol key) {
    if (!_sealed) {
      throw new InjectorError('Can\'t unseal a non-sealed mapping.');
    }
    if (key != _sealKey) {
      throw new InjectorError('Can\'t unseal mapping without the correct key.');
    }

    _sealed = false;
    _sealKey = null;
    return this;
  }

  InjectionMapping toValue(Object value) {
    toProvider(new ValueProvider(value));
    return this;
  }

  InjectionMapping toProvider(IProvider provider) {
    if (_sealed)
      throw new InjectorError('Can\'t change a sealed mapping');

    if (hasProvider && provider != null) {
      print(
          'Warning: Injector already has a mapping for ' + _mappingId + '.\n ' +
              'If you have overridden this mapping intentionally you can use ' +
              '"injector.unmap()" prior to your replacement mapping in order to ' +
              'avoid seeing this message.');
    }

    _creatingInjector.providerMappings[_mappingId] = provider;
    return this;
  }

  IProvider getProvider()
  {
    IProvider provider = _creatingInjector.providerMappings[_mappingId];
    return provider;
  }


}